// Copyright (c) 2015, Mikhail Stolyarov. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library do2template.base;

import 'dart:html';
import 'dart:async';

import 'node_validate.dart';
import 'do2html.dart';

/// Шаблонизатор
class Do2Template {

  /// Асинхроная загрузка HTML шаблона
  /// [url] - http адресс html шаблонна
  /// Возвращает объект [Do2Template]
  static Future loadingHtml(Uri url){
    Completer<Do2Template> template = new Completer();
    HttpRequest.getString(url.toString()).then((String contentHtml){
      template.complete(new Do2Template(contentHtml));
    }).catchError((Error error) {
      print(error.toString());
      template.complete(new Do2Template('<do2template></do2template>'));
    });
    return template.future;
  }

  /// Регистрируем свои атрибуты и элементы
  static final NodeValidatorBuilder nodeValidator = new NodeValidatorBuilder.common()..add(new Do2NodeValidator());

  /// Текстовое представление шаблона
  final String html;

  /// Функции для создания ссылок и небольшой обработки
  List<Function> bind = new List();

  Do2Template(String template): html = template;

  /// Возвращает экземпляр шаблона объектом [Do2Html]
  Do2Html getTemplate(){
    Element element = new Element.html(html, validator:nodeValidator);
    if(element.nodeName == 'DO2TEMPLATE') {
      _bind(element);
      return new Do2Html(element);
    }
    return new Do2Html(new DivElement());
  }

  /// Вызываем функции создания ссылок
  void _bind(Element element){
    bind.forEach((Function func){
      func(element);
    });
  }
}
